/*
    prefs.cpp  -  Floodcast preferences widget

    Copyright (c) 2002 by Ryan Cumming
    Copyright (c) 2004 by Stefan Gehn <metz AT gehn.net>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "server.h"
#include "prefs.h"
#include "plugin.h"
#include "prefswidget.h"
#include "floodcastconfig.h"

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qspinbox.h>
#include <kdebug.h>
#include <klineedit.h>
#include <klocale.h>
#include <kurlrequester.h>

Prefs::Prefs(FloodcastPlugin *parent)
	: CModule(i18n("Floodcast"),
		i18n("Configure the Noatun Icecast Server"), "network", parent),
	mModified(false)
{
	QVBoxLayout *top = new QVBoxLayout(this);
	mWidget = new PrefsWidget(this, "mWidget");
	top->addWidget(mWidget);

	connect(mWidget->kcfg_Mode, SIGNAL(clicked(int)),
		this, SLOT(slotModeChanged(int)));
	connect(mWidget->kcfg_ForwardHost, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_ForwardPort, SIGNAL(valueChanged(int)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_ForwardPassword, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_ForwardType, SIGNAL(activated(int)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_ServerPort, SIGNAL(valueChanged(int)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_ServerMaxUsers, SIGNAL(valueChanged(int)),
		this, SLOT(slotModified()));

	connect(mWidget->kcfg_LamePath, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_Samplerate, SIGNAL(activated(int)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_Bitrate, SIGNAL(valueChanged(int)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_ErrorProtection, SIGNAL(clicked()),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_Mono, SIGNAL(clicked()),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_HighQualityEncoding, SIGNAL(clicked()),
		this, SLOT(slotModified()));

	connect(mWidget->kcfg_StreamTitle, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_StreamGenre, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_StreamURL, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_StreamTrackname, SIGNAL(textChanged(const QString &)),
		this, SLOT(slotModified()));
	connect(mWidget->kcfg_StreamUseTags, SIGNAL(clicked()),
		this, SLOT(slotUseTagsClicked()));

}


void Prefs::reopen()
{
	kdDebug(66666) << k_funcinfo << endl;

	FloodcastConfig::self()->readConfig();

	mWidget->kcfg_Mode->setButton(FloodcastConfig::mode());
	slotModeChanged(FloodcastConfig::mode());
	mWidget->kcfg_ForwardHost->setText(FloodcastConfig::forwardHost());
	mWidget->kcfg_ForwardPort->setValue(FloodcastConfig::forwardPort());
	mWidget->kcfg_ForwardPassword->setText(FloodcastConfig::forwardPassword());
	mWidget->kcfg_ForwardType->setCurrentItem(FloodcastConfig::forwardType());
	mWidget->kcfg_ServerPort->setValue(FloodcastConfig::serverPort());
	mWidget->kcfg_ServerMaxUsers->setValue(FloodcastConfig::serverMaxUsers());
	mWidget->kcfg_LamePath->setURL(FloodcastConfig::lamePath());
	mWidget->kcfg_Bitrate->setValue(FloodcastConfig::bitrate());
	mWidget->kcfg_Samplerate->setCurrentItem(FloodcastConfig::samplerate());
	mWidget->kcfg_ErrorProtection->setChecked(FloodcastConfig::errorProtection());
	mWidget->kcfg_Mono->setChecked(FloodcastConfig::mono());
	mWidget->kcfg_HighQualityEncoding->setChecked(FloodcastConfig::highQualityEncoding());
	mWidget->kcfg_StreamTitle->setText(FloodcastConfig::streamTitle());
	mWidget->kcfg_StreamGenre->setText(FloodcastConfig::streamGenre());
	mWidget->kcfg_StreamURL->setText(FloodcastConfig::streamURL());
	mWidget->kcfg_StreamUseTags->setChecked(FloodcastConfig::streamUseTags());
	mWidget->kcfg_StreamTrackname->setText(FloodcastConfig::streamTrackname());
	slotUseTagsClicked();

	mModified = false;
}


void Prefs::save()
{
	kdDebug(66666) << k_funcinfo << endl;
	if (!mModified)
	{
		kdDebug(66666) << k_funcinfo << "No changes were made, aborting"<< endl;
		return;
	}

	FloodcastConfig::setMode(mWidget->kcfg_Mode->selectedId());
	FloodcastConfig::setForwardHost(mWidget->kcfg_ForwardHost->text());

	FloodcastConfig::setForwardPort(mWidget->kcfg_ForwardPort->value());
	FloodcastConfig::setForwardPassword(mWidget->kcfg_ForwardPassword->text());
	FloodcastConfig::setForwardType(mWidget->kcfg_ForwardType->currentItem());
	FloodcastConfig::setServerPort(mWidget->kcfg_ServerPort->value());
	FloodcastConfig::setServerMaxUsers(mWidget->kcfg_ServerMaxUsers->value());
	FloodcastConfig::setLamePath(mWidget->kcfg_LamePath->url());
	FloodcastConfig::setBitrate(mWidget->kcfg_Bitrate->value());
	FloodcastConfig::setSamplerate(mWidget->kcfg_Samplerate->currentItem());
	FloodcastConfig::setErrorProtection(mWidget->kcfg_ErrorProtection->isChecked());
	FloodcastConfig::setMono(mWidget->kcfg_Mono->isChecked());
	FloodcastConfig::setHighQualityEncoding(mWidget->kcfg_HighQualityEncoding->isChecked());
	FloodcastConfig::setStreamTitle(mWidget->kcfg_StreamTitle->text());
	FloodcastConfig::setStreamGenre(mWidget->kcfg_StreamGenre->text());
	FloodcastConfig::setStreamURL(mWidget->kcfg_StreamURL->text());
	FloodcastConfig::setStreamUseTags(mWidget->kcfg_StreamUseTags->isChecked());
	FloodcastConfig::setStreamTrackname(mWidget->kcfg_StreamTrackname->text());

	FloodcastConfig::self()->writeConfig();
	mModified = false;
	emit saved();
}

void Prefs::slotModified()
{
	mModified = true;
}

void Prefs::slotUseTagsClicked()
{
	mWidget->kcfg_StreamTrackname->setEnabled(!mWidget->kcfg_StreamUseTags->isChecked());
}

void Prefs::slotModeChanged(int newMode)
{
	bool standalone = (newMode == FloodcastConfig::Standalone);

	mWidget->grpServerMode->setEnabled(standalone);
	mWidget->grpForwardMode->setEnabled(!standalone);
	slotModified();
}

#include "prefs.moc"
